Array

Struct Array 

pub struct Array<ArrayImpl, const NDIM: usize>(/* private fields */);
The basic tuple type defining an array.

Implementations

Source

impl<ArrayImpl, const ADIM: usize> Array<ArrayImpl, ADIM>

pub fn insert_empty_axis<const NDIM: usize>( self, axis_position: AxisPosition, ) -> Array<ArrayAppendAxis<ArrayImpl, ADIM, NDIM>, NDIM>
where NumberType<ADIM>: IsSmallerByOne<NDIM>,

Insert empty axis

Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>

pub fn with_container_type<TypeHint: ContainerTypeRepr + ArrayFromContainerType + RowMajorArrayFromContainerType>( self, ) -> Array<WithEvalType<ArrayImpl, TypeHint, NDIM>, NDIM>

Coerce the array to a specific item type and dimension.

Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>

pub fn into_flat(self) -> Array<ArrayFlatView<ArrayImpl, NDIM>, 1>

Return a flattened 1d view onto the array. The view is flattened in column-major order.

Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>

pub fn into_shared(self) -> ArraySharedRef<ArrayImpl, NDIM>

Create a non thread-safe runtime shared reference from the array

Use borrow and borrow_mut to access the underlying array.

The rational of this function is to obtain a cloneable shared reference structure to the original array with runtime borrow checking when compile-time checking is not sufficient.

This function is not thread-safe. For a thread-safe variant use Array::into_shared_atomic.

pub fn into_shared_atomic(self) -> ArrayAtomicSharedRef<ArrayImpl, NDIM>

Create an atomic thread-safe runtime shared reference from the array Use borrow and borrow_mut to access the underlying array.

The rational of this function is to obtain a cloneable shared reference structure to the original array with runtime borrow checking when compile-time checking is not sufficient.

This function is thread-safe. For a slightly faster non thread-safe variant use Array::into_shared.

Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>

pub fn new(arr: ArrayImpl) -> Self

Instantiate a new array from an ArrayImpl structure.

pub fn imp(&self) -> &ArrayImpl

Return a reference to the implementation.

pub fn imp_mut(&mut self) -> &mut ArrayImpl

Return a mutable reference to the implementation.

pub fn into_imp(self) -> ArrayImpl

Extract the inner implementation.

pub fn r(&self) -> Array<ArrayRef<'_, ArrayImpl, NDIM>, NDIM>

Create an owned reference to the array.

pub fn r_mut(&mut self) -> Array<ArrayRefMut<'_, ArrayImpl, NDIM>, NDIM>

Create an owned mutable reference to the array.

Source

impl<ArrayImpl: Shape<NDIM>, const NDIM: usize> Array<ArrayImpl, NDIM>

pub fn number_of_elements(&self) -> usize

Return the number of elements in the array.

Source

impl<Item: Copy + Default, const NDIM: usize> Array<BaseArray<VectorContainer<Item>, NDIM>, NDIM>

pub fn from_shape(shape: [usize; NDIM]) -> Self

Create a new heap allocated array from a given shape.

pub fn from_shape_and_vec(shape: [usize; NDIM], data: Vec<Item>) -> Self

Create a new heap allocated array by providing a shape and a vector of data.

The number of elements in the vector must be compatible with the given shape. Otherwise, an assertion error is triggered.

Source

impl<Item: Copy + Default, const NDIM: usize, const ALIGNMENT: usize> Array<BaseArray<AlignedVectorContainer<Item, ALIGNMENT>, NDIM>, NDIM>

pub fn from_shape(shape: [usize; NDIM]) -> Self

Create a new heap allocated array from a given shape.

pub fn from_shape_and_avec( shape: [usize; NDIM], data: AVec<Item, ConstAlign<ALIGNMENT>>, ) -> Self

Create a new heap allocated array by providing a shape and a vector of data.

The number of elements in the vector must be compatible with the given shape. Otherwise, an assertion error is triggered.

Source

impl<Item: Copy + Default> Array<BaseArray<VectorContainer<Item>, 2>, 2>

pub fn from_iter_aij<Iter: Iterator<Item = ([usize; 2], Item)>>( shape: [usize; 2], iter: Iter, ) -> DynArray<Item, 2>

Create a new dense matrix with shape shape filled with values from the iterator iter.

Source

impl<Item: Copy + Default, const ALIGNMENT: usize> Array<BaseArray<AlignedVectorContainer<Item, ALIGNMENT>, 2>, 2>

pub fn from_iter_aij<Iter: Iterator<Item = ([usize; 2], Item)>>( shape: [usize; 2], iter: Iter, ) -> AlignedDynArray<Item, 2, ALIGNMENT>

Create a new dense matrix with shape shape filled with values from the iterator iter.

Source

impl<Item: Copy + Default, const NDIM: usize> Array<StridedBaseArray<VectorContainer<Item>, NDIM>, NDIM>

pub fn from_shape_and_stride( shape: [usize; NDIM], stride: [usize; NDIM], ) -> Self

Create a new heap allocated array from a given shape and stride.

Source

impl<Item: Copy + Default, const NDIM: usize> Array<StridedBaseArray<VectorContainer<Item>, NDIM>, NDIM>

pub fn row_major(shape: [usize; NDIM]) -> Self

Create a new heap allocated row-major array.

Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: Shape<NDIM>,

pub fn shape(&self) -> [usize; NDIM]

Return the shape of an array.

Traits:
Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: Shape<NDIM>,

pub fn len(&self) -> usize

Return the length of an array.

For more than one dimension the length is the number of elements.

Traits:
pub fn is_empty(&self) -> bool

Return true if the array is empty.

This is equivalent to at least one dimension being zero.

Traits:
Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandomAccessByValue<NDIM>,

pub unsafe fn get_value_unchecked( &self, multi_index: [usize; NDIM], ) -> ArrayImpl::Item

Get an element by value.

Safety

multi_index must be in range.

Traits
Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandomAccessByRef<NDIM>,

pub unsafe fn get_unchecked( &self, multi_index: [usize; NDIM], ) -> &ArrayImpl::Item

Get an element by reference.

Safety

multi_index must be in range.

Traits
Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandomAccessMut<NDIM>,

pub unsafe fn get_unchecked_mut( &mut self, multi_index: [usize; NDIM], ) -> &mut ArrayImpl::Item

Get an element by mutable reference.

Safety

multi_index must be in range.

Traits
Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: RandomAccessByValue<NDIM>,

pub fn get_value(&self, multi_index: [usize; NDIM]) -> Option<ArrayImpl::Item>

Get an element by value.

Traits
Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: RandomAccessByRef<NDIM>,

pub fn get(&self, multi_index: [usize; NDIM]) -> Option<&ArrayImpl::Item>

Get an element by reference.

Traits
Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: RandomAccessMut<NDIM>,

pub fn get_mut( &mut self, multi_index: [usize; NDIM], ) -> Option<&mut ArrayImpl::Item>

Get an element by mutable reference.

Traits
Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: RawAccess,

pub fn data(&self) -> Option<&[ArrayImpl::Item]>

Return the raw data as slice.

Traits
Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: RawAccessMut,

pub fn data_mut(&mut self) -> Option<&mut [ArrayImpl::Item]>

Return the raw data as slice.

Traits
Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: Stride<NDIM>,

pub fn stride(&self) -> [usize; NDIM]

Return the stride.

Traits
Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: Stride<NDIM> + Shape<NDIM>,

pub fn memory_layout(&self) -> MemoryLayout

Return the memory layout

The possible memory layouts are defined in MemoryLayout.

pub fn is_contiguous(&self) -> bool

Return true of the memory layout is contiguous, otherwise false.

A memory layout is contiguous if it is either row or column major.

Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: ResizeInPlace<NDIM>,

pub fn resize_in_place(&mut self, shape: [usize; NDIM])

Resize the array to the new shape.

The content of the array will be lost upon resizing.

Traits
Source

impl<'a, Item, const NDIM: usize> Array<BaseArray<SliceContainer<'a, Item>, NDIM>, NDIM>
where Item: Copy + Default,

pub fn from_shape(slice: &'a [Item], shape: [usize; NDIM]) -> Self

Create a new array from slice with a given shape.

Source

impl<'a, Item, const NDIM: usize> Array<BaseArray<SliceContainerMut<'a, Item>, NDIM>, NDIM>
where Item: Copy + Default,

pub fn from_shape(slice: &'a mut [Item], shape: [usize; NDIM]) -> Self

Create a new array from mutable slice with a given shape.

Source

impl<'a, Item, const NDIM: usize> Array<StridedBaseArray<SliceContainer<'a, Item>, NDIM>, NDIM>
where Item: Copy + Default,

pub fn from_shape_and_stride( slice: &'a [Item], shape: [usize; NDIM], stride: [usize; NDIM], ) -> Self

Create a new array from slice with a given shape and stride.

Source

impl<'a, Item, const NDIM: usize> Array<StridedBaseArray<SliceContainerMut<'a, Item>, NDIM>, NDIM>
where Item: Copy + Default,

pub fn from_shape_and_stride( slice: &'a mut [Item], shape: [usize; NDIM], stride: [usize; NDIM], ) -> Self

Create a new array from slice with a given shape and stride.

Source

impl<Item, const NDIM: usize> Array<BaseArray<VectorContainer<Item>, NDIM>, NDIM>
where Item: Copy + Default,

pub fn new_from<ArrayImpl>(arr: &Array<ArrayImpl, NDIM>) -> Self
where ArrayImpl: UnsafeRandom1DAccessByValue<Item = Item> + Shape<NDIM>,

Create a new array and fill with values from arr.

Source

impl<Item, const NDIM: usize> Array<StridedBaseArray<VectorContainer<Item>, NDIM>, NDIM>
where Item: Copy + Default,

pub fn new_from<ArrayImpl>( stride: [usize; NDIM], arr: &Array<ArrayImpl, NDIM>, ) -> Self
where ArrayImpl: UnsafeRandom1DAccessByValue<Item = Item> + Shape<NDIM>,

Create a new strided array with stride and fill with values from arr.

pub fn row_major_from<ArrayImpl>(arr: &Array<ArrayImpl, NDIM>) -> Self
where ArrayImpl: UnsafeRandom1DAccessByValue<Item = Item> + Shape<NDIM>,

Create a new row-major array from existing array arr.

Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandom1DAccessByValue + Shape<NDIM>,

pub fn iter_value(&self) -> ArrayDefaultIteratorByValue<'_, ArrayImpl, NDIM>

Iterate through the array by value.

The iterator always proceeds in column-major order.

Traits
Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandom1DAccessByRef + Shape<NDIM>,

pub fn iter_ref(&self) -> ArrayDefaultIteratorByRef<'_, ArrayImpl, NDIM>

Iterate through the array by reference.

The iterator always proceeds in column-major order.

Traits
Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandom1DAccessMut + Shape<NDIM>,

pub fn iter_mut(&mut self) -> ArrayDefaultIteratorMut<'_, ArrayImpl, NDIM>

Iterate through the array by mutable reference.

The iterator always proceeds in column-major order.

Traits
Source

impl<Item, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandomAccessByValue<NDIM, Item = Item> + Shape<NDIM>,

pub fn diag_iter_value(&self) -> ArrayDiagIteratorByValue<'_, ArrayImpl, NDIM>

Iterate through the diagonal of the array by value.

Traits
Source

impl<Item, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandomAccessByRef<NDIM, Item = Item> + Shape<NDIM>,

pub fn diag_iter_ref(&self) -> ArrayDiagIteratorByRef<'_, ArrayImpl, NDIM>

Iterate through the diagonal of the array by reference.

Traits
Source

impl<Item, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandomAccessMut<NDIM, Item = Item> + Shape<NDIM>,

pub fn diag_iter_mut(&mut self) -> ArrayDiagIteratorMut<'_, ArrayImpl, NDIM>

Iterate through the diagonal of the array by mutable reference.

Traits
Source

impl<Item, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandom1DAccessMut<Item = Item> + Shape<NDIM>,

pub fn fill_from<ArrayImplOther>(&mut self, other: &Array<ArrayImplOther, NDIM>)
where ArrayImplOther: UnsafeRandom1DAccessByValue<Item = Item> + Shape<NDIM>,

Fill from another array.

#Traits

Source

impl<Item, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where Const<NDIM>: ToUInt, <Const<NDIM> as ToUInt>::Output: Unsigned + NonZero, ArrayImpl: UnsafeRandom1DAccessByValue<Item = Item> + UnsafeRandomAccessByValue<NDIM, Item = Item> + Shape<NDIM>,

pub fn multislice_iter<const SDIM: usize, const OUTDIM: usize>( &self, axes: [usize; SDIM], ) -> MultisliceIterator<'_, Item, ArrayImpl, NDIM, SDIM, OUTDIM>
where Const<SDIM>: ToUInt, Const<OUTDIM>: ToUInt, <Const<SDIM> as ToUInt>::Output: Unsigned + NonZero, <Const<OUTDIM> as ToUInt>::Output: Unsigned + NonZero, NumberType<SDIM>: IsSmallerThan<NDIM>, PInt<U<OUTDIM>>: Add<PInt<U<SDIM>>, Output = PInt<U<NDIM>>>, PInt<U<NDIM>>: Sub<PInt<U<SDIM>>>,

Create a new multislice iterator.

Given an array arr with dimensions [2, 3, 4, 5]. Calling self.multislice_iter with axes = [0, 2] returns an iterator over values of the type (indices, slice), where indices is a [2; usize] array taking all possible axis indices in axes 0 and 2, and slice is the output of arr.multislice([0, 2], [indices]).

Source

impl<Item, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where Const<NDIM>: ToUInt, <Const<NDIM> as ToUInt>::Output: Unsigned + NonZero, ArrayImpl: UnsafeRandom1DAccessMut<Item = Item> + UnsafeRandomAccessMut<NDIM, Item = Item> + Shape<NDIM>,

pub fn multislice_iter_mut<const SDIM: usize, const OUTDIM: usize>( &mut self, axes: [usize; SDIM], ) -> MultisliceIteratorMut<'_, Item, ArrayImpl, NDIM, SDIM, OUTDIM>
where Const<SDIM>: ToUInt, Const<OUTDIM>: ToUInt, <Const<SDIM> as ToUInt>::Output: Unsigned + NonZero, <Const<OUTDIM> as ToUInt>::Output: Unsigned + NonZero, NumberType<SDIM>: IsSmallerThan<NDIM>, PInt<U<OUTDIM>>: Add<PInt<U<SDIM>>, Output = PInt<U<NDIM>>>, PInt<U<NDIM>>: Sub<PInt<U<SDIM>>>,

Create a new mutable multislice iterator.

Given an array arr with dimensions [2, 3, 4, 5]. Calling self.multislice_iter_mut with axes = [0, 2] returns an iterator over values of the type (indices, slice), where indices is a [2; usize] array taking all possible axis indices in axes 0 and 2, and slice is the output of arr.multislice([0, 2], [indices]).

Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandom1DAccessMut + Shape<NDIM>,

pub fn fill_from_iter<Iter>(&mut self, iter: Iter)
where Iter: Iterator<Item = ArrayImpl::Item>,

Fill from an iterator.

Note that the array only fills as many values as the other iterator provides.

Traits
Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: ResizeInPlace<NDIM> + Shape<NDIM> + UnsafeRandom1DAccessMut,

pub fn fill_from_resize<ArrayImplOther>( &mut self, other: &Array<ArrayImplOther, NDIM>, )
where ArrayImplOther: Shape<NDIM> + UnsafeRandom1DAccessByValue<Item = ArrayImpl::Item>,

Fill from another array and resize if necessary.

This method is especially useful together with empty_array.

#Traits

Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandom1DAccessMut + Shape<NDIM>,

pub fn fill_with_value(&mut self, value: ArrayImpl::Item)

Fill array with a given value.

Traits
Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandom1DAccessMut + Shape<NDIM>,

pub fn set_zero(&mut self)

Set the elements of the array to zero.

Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandom1DAccessMut + UnsafeRandomAccessMut<NDIM> + Shape<NDIM>, ArrayImpl::Item: One,

pub fn set_identity(&mut self)

Set all off-diagonal elements to zero and the diagonal to one.

Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandomAccessByValue<NDIM> + Shape<NDIM>, ArrayImpl::Item: Sum,

pub fn trace(&self) -> ArrayImpl::Item

Compute the sum of the diagonal values of the array.

Note: The Item type must support the trait std::iter::Sum.

Traits
Source

impl<Item, ArrayImpl> Array<ArrayImpl, 1>
where ArrayImpl: UnsafeRandom1DAccessByValue<Item = Item> + Shape<1>, ArrayImpl::Item: Conj<Output = Item> + Mul<Output = ArrayImpl::Item> + Add<Output = Item>,

pub fn inner<ArrayImplOther>( &self, other: &Array<ArrayImplOther, 1>, ) -> Option<Item>
where ArrayImplOther: UnsafeRandom1DAccessByValue<Item = Item> + Shape<1>,

Compute the inner product of two 1d arrays.

The elements of other are taken as conjugate.

Returns None if the arrays are empty.

Note: The Item type must support the traits Conj, std::iter::Sum and std::ops::Mul.

Traits
Source

impl<Item, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandom1DAccessByValue<Item = Item> + Shape<NDIM>, Item: Abs, <Item as Abs>::Output: Max<Output = <Item as Abs>::Output>,

pub fn max_abs(&self) -> Option<<Item as Abs>::Output>

Compute the maximum absolute value over all elements.

Note: The item type must support Abs and the output of Abs must support Max.

The function returns None if the array is empty.

Traits
Source

impl<Item, ArrayImpl> Array<ArrayImpl, 1>
where ArrayImpl: UnsafeRandom1DAccessByValue<Item = Item> + Shape<1>, Item: Abs, <Item as Abs>::Output: Add<Output = <Item as Abs>::Output>,

pub fn norm_1(&self) -> Option<<Item as Abs>::Output>

Compute the 1-norm of a 1d array.

The function returns None if the array is empty.

Note: The item type must support Abs and the output of Abs must support std::ops::Add.

Traits
Source

impl<Item, ArrayImpl> Array<ArrayImpl, 1>
where ArrayImpl: UnsafeRandom1DAccessByValue<Item = Item> + Shape<1>, Item: AbsSquare, <Item as AbsSquare>::Output: Sqrt<Output = <Item as AbsSquare>::Output> + Add<Output = <Item as AbsSquare>::Output>,

pub fn norm_2(&self) -> Option<<Item as AbsSquare>::Output>

Compute the 2-norm of a 1d array.

Note: The item type must support AbsSquare. The output of AbsSquare must support std::ops::Add and Sqrt.

Traits
Source

impl<Item, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandom1DAccessByValue<Item = Item> + Shape<NDIM>, Item: AbsSquare, <Item as AbsSquare>::Output: Sqrt<Output = <Item as AbsSquare>::Output> + Add<Output = <Item as AbsSquare>::Output> + Copy + Default,

pub fn norm_fro(&self) -> Option<<Item as AbsSquare>::Output>

Compute the Frobenius-Norm of a nd array.

Note: The item type must support AbsSquare. The output of AbsSquare must support std::ops::Add, Sqrt, Copy, and Default.

Traits
Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandom1DAccessByValue + Shape<NDIM>,

pub fn into_type<T>( self, ) -> Array<ArrayUnaryOperator<ArrayImpl::Item, T, ArrayImpl, fn(ArrayImpl::Item) -> T, NDIM>, NDIM>
where ArrayImpl::Item: Into<T>,

Convert an array into the new item type T.

Note: It is required that ArrayImpl::Item: Into<T>.

Traits
Source

impl<ArrayImpl> Array<ArrayImpl, 2>
where ArrayImpl: Shape<2> + UnsafeRandomAccessByValue<2>,

pub fn col_iter(&self) -> ColIterator<'_, ArrayImpl, 2>

Return a column iterator for a 2d array.

Traits
Source

impl<ArrayImpl> Array<ArrayImpl, 2>
where ArrayImpl: Shape<2> + UnsafeRandomAccessMut<2>,

pub fn col_iter_mut(&mut self) -> ColIteratorMut<'_, ArrayImpl, 2>

Return a mutable column iterator for a 2d array.

Traits
Source

impl<ArrayImpl> Array<ArrayImpl, 2>
where ArrayImpl: Shape<2> + UnsafeRandomAccessByValue<2>,

pub fn row_iter(&self) -> RowIterator<'_, ArrayImpl, 2>

Return a row iterator for a 2d array.

Traits
Source

impl<ArrayImpl> Array<ArrayImpl, 2>
where ArrayImpl: Shape<2> + UnsafeRandomAccessByValue<2>,

pub fn row_iter_mut(&mut self) -> RowIteratorMut<'_, ArrayImpl, 2>

Return a mutable row iterator for a 2d array.

Traits
Source

impl<ArrayImpl> Array<ArrayImpl, 2>
where ArrayImpl: UnsafeRandom1DAccessByValue + Shape<2>,

pub fn iter_aij_value( &self, ) -> impl Iterator<Item = ([usize; 2], ArrayImpl::Item)> + '_

Return an iterator of the form (i, j, data).

Here, i is the row, j is the column, and data is the associated element returned by value.

Traits
Source

impl<ArrayImpl> Array<ArrayImpl, 2>
where ArrayImpl: UnsafeRandom1DAccessByRef + Shape<2>,

pub fn iter_aij_ref( &self, ) -> impl Iterator<Item = ([usize; 2], &ArrayImpl::Item)> + '_

Return an iterator of the form (i, j, &data).

Here, i is the row, j is the column, and &data is the reference to the associated element.

Traits
Source

impl<ArrayImpl> Array<ArrayImpl, 2>
where ArrayImpl: UnsafeRandom1DAccessMut + Shape<2>,

pub fn iter_aij_mut( &mut self, ) -> impl Iterator<Item = ([usize; 2], &mut ArrayImpl::Item)> + '_

Return an iterator of the form (i, j, &mut data).

Here, i is the row, j is the column, and &data is the mutable reference to the associated element.

Traits
Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>

pub fn cast<Target>(self) -> Array<ArrayCast<Target, ArrayImpl, NDIM>, NDIM>

Cast array to type T.

The cast is done through num::cast::cast and source and target types need to support casting through that function.

Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>

pub fn coerce_dim<const CDIM: usize>( self, ) -> RlstResult<Array<CoerceArray<ArrayImpl, NDIM, CDIM>, CDIM>>

Coerce the array to a specific dimension.

This is useful to coerce from a generic dimension parameter to a specific number of dimensions.

Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: Shape<NDIM>,

pub fn permute_axes( self, permutation: [usize; NDIM], ) -> Array<ArrayTranspose<ArrayImpl, NDIM>, NDIM>

Permute axes of an array.

The permutation gives the new ordering of the axes.

pub fn transpose(self) -> Array<ArrayTranspose<ArrayImpl, NDIM>, NDIM>

Transpose an array.

The transpose of an n-dimensional array reverses the order of the axes.

Source

impl<Item, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where Item: RandScalar + RlstScalar, ArrayImpl: UnsafeRandom1DAccessMut<Item = Item> + Shape<NDIM>, StandardNormal: Distribution<Item::Real>, StandardUniform: Distribution<Item::Real>,

pub fn fill_from_standard_normal<R: Rng>(&mut self, rng: &mut R)

Fill an array with normally distributed random numbers.

Traits
pub fn fill_from_equally_distributed<R: Rng>(&mut self, rng: &mut R)

Fill an array with equally distributed random numbers.

Traits
pub fn fill_from_seed_equally_distributed(&mut self, seed: usize)

Fill an array with equally distributed random numbers using a given seed.

Traits
pub fn fill_from_seed_normally_distributed(&mut self, seed: usize)

Fill an array with normally distributed random numbers using a given seed.

Traits
Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>

pub fn reverse_axis( self, axis: usize, ) -> Array<ReverseAxis<ArrayImpl, NDIM>, NDIM>

Reverse a single axis of the array.

Source

impl<ArrayImpl, const ADIM: usize> Array<ArrayImpl, ADIM>
where ArrayImpl: Shape<ADIM>,

pub fn slice<const OUTDIM: usize>( self, axis: usize, index: usize, ) -> Array<ArraySlice<ArrayImpl, ADIM, OUTDIM>, OUTDIM>
where NumberType<ADIM>: IsGreaterByOne<OUTDIM>, NumberType<OUTDIM>: IsGreaterZero,

Create a slice from a given array.

Consider an array arr with shape [a0, a1, a2, a3, ...]. The function call arr.slice(2, 3) returns a one dimension smaller array indexed by [a0, a1, 3, a3, ...]. Hence, the dimension 2 has been fixed to always have the value 3.

Examples

If arr is a matrix then the first column of the matrix is obtained from arr.slice(1, 0), while the third row of the matrix is obtained from arr.slice(0, 2).

pub fn multislice<const SDIM: usize, const OUTDIM: usize>( self, axes: [usize; SDIM], indices: [usize; SDIM], ) -> Array<Multislice<ArrayImpl, ADIM, SDIM, OUTDIM>, OUTDIM>
where NumberType<SDIM>: IsSmallerThan<ADIM>, NumberDifference<ADIM, SDIM>: SubtractEqual<OUTDIM>,

Create a multislice from a given array.

A multislice works like Array::slice but allows the slicing along multiple dimensions at the same time. The axes array determines which axes are fixed and the indices array determines the value in each fixed axis. The dimension SDIM is the number of axes along which we slice. The parameter OUTDIM is the number of dimensions after slicing.

Source

impl<ArrayImpl> Array<ArrayImpl, 2>
where ArrayImpl: Shape<2>,

pub fn row(self, row_index: usize) -> Array<ArraySlice<ArrayImpl, 2, 1>, 1>

Return the row with index row_index from a two-dimensional array.

pub fn col(self, col_index: usize) -> Array<ArraySlice<ArrayImpl, 2, 1>, 1>

Return the column with index col_index from a two-dimensional array.

Source

impl<ArrayImpl: Shape<NDIM>, const NDIM: usize> Array<ArrayImpl, NDIM>

pub fn into_subview( self, offset: [usize; NDIM], shape: [usize; NDIM], ) -> Array<ArraySubView<ArrayImpl, NDIM>, NDIM>

Move the array into a subview specified by an offset and shape of the subview.

The offset is the starting index of the subview and the shape is the number of indices in each dimension of the subview.

Source

impl<ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>

pub fn unary_op<OpItem, OpTarget, Op: Fn(OpItem) -> OpTarget>( self, op: Op, ) -> Array<ArrayUnaryOperator<OpItem, OpTarget, ArrayImpl, Op, NDIM>, NDIM>

Create a new array by applying the unitary operator op to each element of self.

Source

impl<Item: Default + Copy + Mul<Output = Item>, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn scalar_mul( self, scalar: Item, ) -> Array<ArrayScalarMult<Item, ArrayImpl, NDIM>, NDIM>

Multiple the array with a given scalar.

Note: The Item type must support std::ops::Mul.

Source

impl<Item: Conj, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn conj( self, ) -> Array<ArrayUnaryOperator<Item, <Item as Conj>::Output, ArrayImpl, fn(Item) -> <Item as Conj>::Output, NDIM>, NDIM>

Componentwise apply conj to the array.

Source

impl<Item: Abs, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn abs( self, ) -> Array<ArrayUnaryOperator<Item, <Item as Abs>::Output, ArrayImpl, fn(Item) -> <Item as Abs>::Output, NDIM>, NDIM>

Componentwise apply abs to the array.

Source

impl<Item: Square, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn square( self, ) -> Array<ArrayUnaryOperator<Item, <Item as Square>::Output, ArrayImpl, fn(Item) -> <Item as Square>::Output, NDIM>, NDIM>

Componentwise apply square to the array.

Source

impl<Item: AbsSquare, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn abs_square( self, ) -> Array<ArrayUnaryOperator<Item, <Item as AbsSquare>::Output, ArrayImpl, fn(Item) -> <Item as AbsSquare>::Output, NDIM>, NDIM>

Componentwise apply abs_square to the array.

Source

impl<Item: Sqrt, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn sqrt( self, ) -> Array<ArrayUnaryOperator<Item, <Item as Sqrt>::Output, ArrayImpl, fn(Item) -> <Item as Sqrt>::Output, NDIM>, NDIM>

Componentwise apply sqrt to the array.

Source

impl<Item: Exp, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn exp( self, ) -> Array<ArrayUnaryOperator<Item, <Item as Exp>::Output, ArrayImpl, fn(Item) -> <Item as Exp>::Output, NDIM>, NDIM>

Componentwise apply exp to the array.

Source

impl<Item: Ln, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn ln( self, ) -> Array<ArrayUnaryOperator<Item, <Item as Ln>::Output, ArrayImpl, fn(Item) -> <Item as Ln>::Output, NDIM>, NDIM>

Componentwise apply ln to the array.

Source

impl<Item: Recip, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn recip( self, ) -> Array<ArrayUnaryOperator<Item, <Item as Recip>::Output, ArrayImpl, fn(Item) -> <Item as Recip>::Output, NDIM>, NDIM>

Componentwise apply recip to the array.

Source

impl<Item: Sin, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn sin( self, ) -> Array<ArrayUnaryOperator<Item, <Item as Sin>::Output, ArrayImpl, fn(Item) -> <Item as Sin>::Output, NDIM>, NDIM>

Componentwise apply sin to the array.

Source

impl<Item: Cos, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn cos( self, ) -> Array<ArrayUnaryOperator<Item, <Item as Cos>::Output, ArrayImpl, fn(Item) -> <Item as Cos>::Output, NDIM>, NDIM>

Componentwise apply cos to the array.

Source

impl<Item: Tan, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn tan( self, ) -> Array<ArrayUnaryOperator<Item, <Item as Tan>::Output, ArrayImpl, fn(Item) -> <Item as Tan>::Output, NDIM>, NDIM>

Componentwise apply tan to the array.

Source

impl<Item: Asin, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn asin( self, ) -> Array<ArrayUnaryOperator<Item, <Item as Asin>::Output, ArrayImpl, fn(Item) -> <Item as Asin>::Output, NDIM>, NDIM>

Componentwise apply asin to the array.

Source

impl<Item: Acos, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn acos( self, ) -> Array<ArrayUnaryOperator<Item, <Item as Acos>::Output, ArrayImpl, fn(Item) -> <Item as Acos>::Output, NDIM>, NDIM>

Componentwise apply acos to the array.

Source

impl<Item: Atan, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn atan( self, ) -> Array<ArrayUnaryOperator<Item, <Item as Atan>::Output, ArrayImpl, fn(Item) -> <Item as Atan>::Output, NDIM>, NDIM>

Componentwise apply atan to the array.

Source

impl<Item: Sinh, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn sinh( self, ) -> Array<ArrayUnaryOperator<Item, <Item as Sinh>::Output, ArrayImpl, fn(Item) -> <Item as Sinh>::Output, NDIM>, NDIM>

Componentwise apply sinh to the array.

Source

impl<Item: Cosh, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn cosh( self, ) -> Array<ArrayUnaryOperator<Item, <Item as Cosh>::Output, ArrayImpl, fn(Item) -> <Item as Cosh>::Output, NDIM>, NDIM>

Componentwise apply cosh to the array.

Source

impl<Item: Tanh, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn tanh( self, ) -> Array<ArrayUnaryOperator<Item, <Item as Tanh>::Output, ArrayImpl, fn(Item) -> <Item as Tanh>::Output, NDIM>, NDIM>

Componentwise apply tanh to the array.

Source

impl<Item: Asinh, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn asinh( self, ) -> Array<ArrayUnaryOperator<Item, <Item as Asinh>::Output, ArrayImpl, fn(Item) -> <Item as Asinh>::Output, NDIM>, NDIM>

Componentwise apply asinh to the array.

Source

impl<Item: Acosh, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn acosh( self, ) -> Array<ArrayUnaryOperator<Item, <Item as Acosh>::Output, ArrayImpl, fn(Item) -> <Item as Acosh>::Output, NDIM>, NDIM>

Componentwise apply acosh to the array.

Source

impl<Item: Atanh, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: BaseItem<Item = Item>,

pub fn atanh( self, ) -> Array<ArrayUnaryOperator<Item, <Item as Atanh>::Output, ArrayImpl, fn(Item) -> <Item as Atanh>::Output, NDIM>, NDIM>

Componentwise apply atanh to the array.

Source

impl<Item, ArrayImpl> Array<ArrayImpl, 2>
where ArrayImpl: RawAccess<Item = Item> + Stride<2> + Shape<2>, Item: Gemm + Copy + Default + One + Zero,

pub fn dot<ArrayImplOther, const NDIM: usize>( &self, other: &Array<ArrayImplOther, NDIM>, ) -> DynArray<Item, NDIM>
where ArrayImplOther: RawAccess<Item = Item> + Shape<NDIM> + Stride<NDIM>,

Compute the matrix-matrix product of self with other.

Source

impl<Item, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandomAccessMut<NDIM, Item = Item>, Item: Copy,

pub unsafe fn swap_unsafe(&mut self, pos1: [usize; NDIM], pos2: [usize; NDIM])

Swap the elements at the positions pos1 and pos2

Safety

pos1 and pos2 must not be out of bounds.

Source

impl<Item, ArrayImpl, const NDIM: usize> Array<ArrayImpl, NDIM>
where ArrayImpl: UnsafeRandomAccessMut<NDIM, Item = Item> + Shape<NDIM>, Item: Copy,

pub fn swap( &mut self, pos1: [usize; NDIM], pos2: [usize; NDIM], ) -> RlstResult<()>

Swap the elements at the positions pos1 and pos2.

Returns an error if either pos1 or pos2 is out of bounds.

Source

impl<Item, const NDIM: usize> Array<BaseArray<ArrayContainer<Item, 1>, NDIM>, NDIM>
where Item: Copy + Default,

pub fn one_element_array() -> Self

Create an empty 1-element array of the given dimension.