SliceArrayMut

Type Alias SliceArrayMut 

pub type SliceArrayMut<'a, Item, const NDIM: usize> = Array<BaseArray<SliceContainerMut<'a, Item>, NDIM>, NDIM>;
A mutable dynamically allocated array from a data slice.

Aliased Type

pub struct SliceArrayMut<'a, Item, const NDIM: usize>(/* private fields */);

Implementations

Source

impl<'a, Item, const NDIM: usize> SliceArrayMut<'a, Item, NDIM>
where Item: Copy + Default,

pub fn from_shape(slice: &'a mut [Item], shape: [usize; NDIM]) -> Self

Create a new array from mutable slice with a given shape.