SimdFor

Struct SimdFor 

pub struct SimdFor<T, S> {
    pub simd: S,
    /* private fields */
}
A simplified wrapper to call into Simd operations in a type and architecture independent way.

Fields

simd: S

Architecture specific Simd information.

Implementations

Source

impl<T: RlstSimd, S: Simd> SimdFor<T, S>

pub fn new(simd: S) -> Self

Instantiate a new Simd object.

pub fn simd_vector_width(self) -> usize

Length of a Simd vector.

pub fn splat(self, value: T) -> T::Scalars<S>

Copy a scalar across all Simd lanes.

pub fn cmp_eq(self, lhs: T::Scalars<S>, rhs: T::Scalars<S>) -> T::Mask<S>

Compare for equality.

pub fn cmp_lt(self, lhs: T::Scalars<S>, rhs: T::Scalars<S>) -> T::Mask<S>

Compare for less than.

pub fn cmp_le(self, lhs: T::Scalars<S>, rhs: T::Scalars<S>) -> T::Mask<S>

Compare for less equal.

pub fn select( self, mask: T::Mask<S>, if_true: T::Scalars<S>, if_false: T::Scalars<S>, ) -> T::Scalars<S>

Select based on bitmask.

pub fn neg(self, value: T::Scalars<S>) -> T::Scalars<S>

Negate input.

pub fn add(self, lhs: T::Scalars<S>, rhs: T::Scalars<S>) -> T::Scalars<S>

Add two Simd vectors.

pub fn sub(self, lhs: T::Scalars<S>, rhs: T::Scalars<S>) -> T::Scalars<S>

Subtract two Simd vectors.

pub fn mul(self, lhs: T::Scalars<S>, rhs: T::Scalars<S>) -> T::Scalars<S>

Multiply two Simd vectors.

pub fn mul_add( self, lhs: T::Scalars<S>, rhs: T::Scalars<S>, acc: T::Scalars<S>, ) -> T::Scalars<S>

Multiply lhs and rhs and add to acc.

pub fn div(self, lhs: T::Scalars<S>, rhs: T::Scalars<S>) -> T::Scalars<S>

Divide two Simd vectors.

pub fn sin_cos(self, value: T::Scalars<S>) -> (T::Scalars<S>, T::Scalars<S>)

Compute the sine and cosine of two Simd vectors.

pub fn exp(self, value: T::Scalars<S>) -> T::Scalars<S>

Compute the base e exponential of a Simd vector.

pub fn sqrt(self, value: T::Scalars<S>) -> T::Scalars<S>

Compute the square root of a Simd vector.

pub fn approx_recip(self, value: T::Scalars<S>) -> T::Scalars<S>

Compute an approximate inverse of a Simd vector.

pub fn approx_recip_sqrt(self, value: T::Scalars<S>) -> T::Scalars<S>

Compute an approximate inverse of a Simd vector.

pub fn reduce_add(self, value: T::Scalars<S>) -> T

Sum the elements of a Simd vector.

pub fn deinterleave<const N: usize>( self, value: [T::Scalars<S>; N], ) -> [T::Scalars<S>; N]

Deinterleave N Simd vectors.

pub fn interleave<const N: usize>( self, value: [T::Scalars<S>; N], ) -> [T::Scalars<S>; N]

Interleave N Simd vectors.