pub trait UnsafeRandom1DAccessMut: BaseItem {
// Required method
unsafe fn get_1d_unchecked_mut(&mut self, index: usize) -> &mut Self::Item;
}Expand description
This trait provides unsafe mutable access to the underlying data using a 1D index.
Required Methods
Sourceunsafe fn get_1d_unchecked_mut(&mut self, index: usize) -> &mut Self::Item
unsafe fn get_1d_unchecked_mut(&mut self, index: usize) -> &mut Self::Item
Return a mutable reference to the element at position determined by index.
Safety
index must not be out of bounds.