pub type StridedDynArray<Item, const NDIM: usize> = Array<StridedBaseArray<VectorContainer<Item>, NDIM>, NDIM>;Expand description
A basic dynamically allocated array with a given stride.
Aliased Type
pub struct StridedDynArray<Item, const NDIM: usize>(/* private fields */);Implementations
Sourceimpl<Item, const NDIM: usize> StridedDynArray<Item, NDIM>
impl<Item, const NDIM: usize> StridedDynArray<Item, NDIM>
Sourcepub fn new_from<ArrayImpl>(
stride: [usize; NDIM],
arr: &Array<ArrayImpl, NDIM>,
) -> Selfwhere
ArrayImpl: UnsafeRandom1DAccessByValue<Item = Item> + Shape<NDIM>,
pub fn new_from<ArrayImpl>(
stride: [usize; NDIM],
arr: &Array<ArrayImpl, NDIM>,
) -> Selfwhere
ArrayImpl: UnsafeRandom1DAccessByValue<Item = Item> + Shape<NDIM>,
Create a new strided array with stride and fill with values from arr.
Sourcepub fn row_major_from<ArrayImpl>(arr: &Array<ArrayImpl, NDIM>) -> Selfwhere
ArrayImpl: UnsafeRandom1DAccessByValue<Item = Item> + Shape<NDIM>,
pub fn row_major_from<ArrayImpl>(arr: &Array<ArrayImpl, NDIM>) -> Selfwhere
ArrayImpl: UnsafeRandom1DAccessByValue<Item = Item> + Shape<NDIM>,
Create a new row-major array from existing array arr.